/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

include("Screen.js");

/**
 * Message screen, for temporarily displaying short messages on the whole
 * screen.
 */
function AlarmScreen() {	
	var fontSize = (system.platform == "windows") ? 8 : 9;
	
	this.message = new Text();
	this.message.width = screen.width - 10;
	this.message.alignment = "center";
	this.message.hOffset = screen.width / 2;
	this.message.vOffset = (screen.height / 2) + 6;
	
	this.snooze = new Text();
	this.snooze.data = "Snooze";
	this.snooze.size = fontSize;
	this.snooze.width = (screen.width - 10) / 2;
	this.snooze.alignment = "center";
	this.snooze.hOffset = (screen.width / 2) / 2;
	this.snooze.vOffset = screen.height - 3;
	this.snooze.onMouseEnter = "Player.instance.alarmScreen.setSelectedButton(Player.instance.alarmScreen.snooze);";
	this.snooze.onMouseUp = "Player.instance.alarmScreen.guideButtonClicked();";

	this.dismiss = new Text();
	this.dismiss.data = "Dismiss";
	this.dismiss.size = fontSize;
	this.dismiss.width = (screen.width - 10) / 2;
	this.dismiss.alignment = "center";
	this.dismiss.hOffset = ((screen.width / 2) / 2) * 3;
	this.dismiss.vOffset = screen.height - 3;
	this.dismiss.onMouseEnter = "Player.instance.alarmScreen.setSelectedButton(Player.instance.alarmScreen.dismiss);";
	this.dismiss.onMouseUp = "Player.instance.alarmScreen.guideButtonClicked();";
	
	this.setSelectedButton(this.dismiss);
	this.updateColors();
	
	this.addSubView(this.message);
	this.addSubView(this.snooze);
	this.addSubView(this.dismiss);
}

AlarmScreen.prototype = new Screen();

AlarmScreen.prototype.setMessage = function(time) {
	this.message.data = "Alarm " + time;
}

AlarmScreen.prototype.setSelectedButton = function(btn) {
	//un highlight the currently selected button
	if (this.selectedButton != null) {
		this.applyShadow(this.selectedButton);
		this.selectedButton.color = this.skinColors("darkColor"); //"#000000";
		this.selectedButton.bgOpacity = 0;
	}

	this.selectedButton = btn;

	if (btn != null)
	{
		this.selectedButton.shadow = null;
		this.selectedButton.color = this.skinColors("highlightText"); //"#FFFFFF";
		this.selectedButton.bgColor = this.skinColors("highlight"); //"#000000";
		this.selectedButton.bgOpacity = 180;
	}
}

AlarmScreen.prototype.upButtonClicked = function() {
	this.setSelectedButton(this.snooze);
}

AlarmScreen.prototype.downButtonClicked = function() {
	this.setSelectedButton(this.dismiss);
}

AlarmScreen.prototype.guideButtonClicked = function() {
	if (this.selectedButton.data == "Snooze") {
		SiriusTuner.instance.stop();
		Player.instance.alarmSnoozeTimer.interval = preferences.snoozeTimeout.value;
		Player.instance.alarmSnoozeTimer.ticking = true;
	} else {
		if (preferences.alarmStopOnDismiss.value == 1) {
			SiriusTuner.instance.stop();
		}
	}

	Player.instance.alarmScreenTimer.ticking = false;
	this.hide();
	Player.instance.activeScreen = Player.instance.mainScreen;
	Player.instance.activeScreen.show();
}

AlarmScreen.prototype.updateChannelData = function() {
	return;
}

AlarmScreen.prototype.updateColors = function() {
	this.applyShadow(this.message);
	this.applyShadow(this.snooze);
	this.applyShadow(this.dismiss);
	
	this.applyFont(this.message);
	this.applyFont(this.snooze);
	this.applyFont(this.dismiss);
	
	this.message.color = this.skinColors("darkColor");
	this.selectedButton.color = this.skinColors("highlightText");
	this.selectedButton.bgColor = this.skinColors("highlight");
	var otherButton = (this.selectedButton.data == "Snooze") ? this.dismiss : this.snooze;
	otherButton.color = this.skinColors("darkColor");
}