/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

include("Screen.js");

/**
 * Message screen, for temporarily displaying short messages on the whole
 * screen.
 */
function ArtistAlertScreen() {
	var fontSize = (system.platform == "windows") ? 8 : 9;
	
	this.title = new Text();
	this.title.data = "Artist Alert";
	this.title.size = fontSize;
	this.title.width = screen.width;
	this.title.style = "bold";
	this.title.bgOpacity = 180;
	this.title.opacity = 255;
	this.title.alignment = "center";
	this.title.hOffset = screen.width / 2;
	this.title.vOffset = this.title.height - 1;
	
	this.message1 = new Text();
	this.message1.size = fontSize;
	this.message1.width = screen.width - 2;
	this.message1.alignment = "center";
	this.message1.scrolling = "off";
	this.message1.truncation = "end";
	this.message1.hOffset = screen.width / 2;
	this.message1.vOffset = 24;

	this.message2 = new Text();
	this.message2.size = fontSize;
	this.message2.width = screen.width - 2;
	this.message2.alignment = "center";
	this.message2.scrolling = "off";
	this.message2.truncation = "end";
	this.message2.hOffset = screen.width / 2;
	this.message2.vOffset = this.message1.vOffset + 10;

	this.message3 = new Text();
	this.message3.size = fontSize;
	this.message3.width = screen.width - 2;
	this.message3.alignment = "center";
	this.message3.hOffset = screen.width / 2;
	this.message3.vOffset = this.message2.vOffset + 10;

	this.message4 = new Text();
	this.message4.size = fontSize;
	this.message4.width = screen.width - 2;
	this.message4.alignment = "center";
	this.message4.hOffset = screen.width / 2;
	this.message4.vOffset = this.message3.vOffset + 10;
	
	this.yes = new Text();
	this.yes.data = "Yes";
	this.yes.size = fontSize;
	this.yes.width = (screen.width - 10) / 2;
	this.yes.alignment = "center";
	this.yes.hOffset = (screen.width / 2) / 2;
	this.yes.vOffset = screen.height - 3;
	this.yes.onMouseEnter = "Player.instance.artistAlertScreen.setSelectedButton(Player.instance.artistAlertScreen.yes);";
	this.yes.onMouseUp = "Player.instance.artistAlertScreen.guideButtonClicked();";

	this.no = new Text();
	this.no.data = "No";
	this.no.size = fontSize;
	this.no.width = (screen.width - 10) / 2;
	this.no.alignment = "center";
	this.no.hOffset = ((screen.width / 2) / 2) * 3;
	this.no.vOffset = screen.height - 3;
	this.no.onMouseEnter = "Player.instance.artistAlertScreen.setSelectedButton(Player.instance.artistAlertScreen.no);";
	this.no.onMouseUp = this.yes.onMouseUp;
	
	this.setSelectedButton(this.yes);
	this.updateColors();
	
	this.addSubView(this.title);
	this.addSubView(this.message1);
	this.addSubView(this.message2);
	this.addSubView(this.message3);
	this.addSubView(this.message4);
	this.addSubView(this.yes);
	this.addSubView(this.no);
}

ArtistAlertScreen.prototype = new Screen();

ArtistAlertScreen.prototype.setMessage = function(ch) {
	this.channel = ch;
	this.message1.data = ch.currentArtist + " - " + ch.currentTitle;
	this.message2.data = "is now playing on " + ch.name + " (" + ch.num + ").";
	this.message3.data = "Do you want to tune";
	this.message4.data = "to the selected station?";
}

ArtistAlertScreen.prototype.setSelectedButton = function(btn) {
	//un highlight the currently selected button
	if (this.selectedButton != null) {
		this.applyShadow(this.selectedButton);
		this.selectedButton.color = this.skinColors("darkColor"); //"#000000";
		this.selectedButton.bgOpacity = 0;
	}

	this.selectedButton = btn;

	if (btn != null)
	{
		this.selectedButton.shadow = null;
		this.selectedButton.color = this.skinColors("highlightText"); //"#FFFFFF";
		this.selectedButton.bgColor = this.skinColors("highlight"); //"#000000";
		this.selectedButton.bgOpacity = 180;
	}
}

ArtistAlertScreen.prototype.upButtonClicked = function() {
	this.setSelectedButton(this.yes);
}

ArtistAlertScreen.prototype.downButtonClicked = function() {
	this.setSelectedButton(this.no);
}

ArtistAlertScreen.prototype.guideButtonClicked = function() {
	if (this.selectedButton.data == "Yes")
		Player.instance.playChannel(this.channel, false);
	Player.instance.artistAlertScreenTimer.ticking = false;
	this.hide();
	Player.instance.activeScreen = Player.instance.mainScreen;
	Player.instance.activeScreen.show();
}

ArtistAlertScreen.prototype.updateChannelData = function() {
	return;
}

ArtistAlertScreen.prototype.show = function() {
	log("Showing ArtistAlertScreen!");
	if (preferences.alertShowBalloon.value == 1) _showBalloon(this.message1.data + "\n" + this.message2.data);
	this.frame.hOffset = screen.hOffset;
	this.frame.vOffset = screen.vOffset;
	if (this.frame.vScrollBar != null) {
		this.frame.vScrollBar.hOffset = screen.hOffset + 151;
		this.frame.vScrollBar.vOffset = screen.vOffset;
	}
	
	this.frame.visible = true;
	if (this.frame.vScrollBar != null) {
		this.frame.vScrollBar.visible = true;
	}
}

ArtistAlertScreen.prototype.updateColors = function() {
	this.applyShadow(this.message1);
	this.applyShadow(this.message2);
	this.applyShadow(this.message3);
	this.applyShadow(this.message4);
	
	this.applyFont(this.title);
	this.applyFont(this.message1);
	this.applyFont(this.message2);
	this.applyFont(this.message3);
	this.applyFont(this.message4);
	this.applyFont(this.yes);
	this.applyFont(this.no);
	
	this.title.color = this.skinColors("headerText");
	this.title.bgColor = this.skinColors("header");
	this.message1.color = this.skinColors("darkColor");
	this.message2.color = this.skinColors("darkColor");
	this.message3.color = this.skinColors("darkColor");
	this.message4.color = this.skinColors("darkColor");
	this.selectedButton.color = this.skinColors("highlightText");
	this.selectedButton.bgColor = this.skinColors("highlight");
	var otherButton = (this.selectedButton.data == "Yes") ? this.no : this.yes;
	otherButton.color = this.skinColors("darkColor");
}