/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

include("Screen.js");

function ChannelGuideScreen(player) {
	this.player = player;
	this.rows = new Array();
	this.selectedRow;
	this.selectedChannel;
	this.scrollBar = new ScrollBar();
}

ChannelGuideScreen.prototype = new Screen();

ChannelGuideScreen.prototype.init = function() {
	log("--> ChannelGuideScreen.init()");
	
	var fontSize = (system.platform == "windows") ? 8 : 9;	
	var useSiriusBackstageData = preferences.useSiriusBackstageData.value;

	// add a row for each channel
	var currentRow;
	var currentChannel = this.player.siriusTuner.firstChannel;
	var currentCategory;
	var i = 0;
	do {
		if (i > 0) {
			currentChannel = currentChannel.nextChannel;
		}
		
		if (currentCategory != currentChannel.category.name) {
			currentCategory = currentChannel.category.name;
			var categoryRow = new Text();
			categoryRow.data = currentCategory;
			categoryRow.size = fontSize;
			categoryRow.width = screen.width;
			categoryRow.style = "bold";
			categoryRow.bgOpacity = 180;
			categoryRow.opacity = 255;
			categoryRow.hOffset = 1;
			categoryRow.vOffset = categoryRow.height * (i + 1);
			this.addSubView(categoryRow);
			i++;
		}

		//var mi = new Array();
		//mi[0] = new MenuItem();
		//mi[0].title = "Show Artist Alert for " + currentChannel.name;
		//mi[0].onSelect = "Player.instance.displayArtistAlert(SiriusTuner.instance.channels[" + currentChannel.num + "]);";
		//mi[1] = new MenuItem();
		//mi[1].title = "-";
		
		log("Adding channel " + currentChannel.num + " to the guide");
		currentRow = new Array();
		currentRow[0] = new Text();
		currentRow[0].data = currentChannel.name;
		currentRow[0].size = fontSize;
		if (useSiriusBackstageData == 1) {
			currentRow[0].width = (screen.width / 2) - 3;
		} else { currentRow[0].width = (screen.width - 2); }
		currentRow[0].opacity = 255;
		currentRow[0].hOffset = 1;
		currentRow[0].vOffset = currentRow[0].height * (i + 1);
		currentRow[0].tooltip = currentChannel.num + ": " + currentChannel.name + " (" + currentChannel.category.name + ") \n" + currentChannel.desc;
		currentRow[0].onMouseUp = "log(Player.instance.channelGuideScreen.channelClicked(" + currentChannel.num + "))";
		currentRow[0].onContextMenu = "Player.instance.channelGuideScreen.buildGuideScreenItemContextMenu(" + currentChannel.num + ");";
		this.addSubView(currentRow[0]);

		currentRow[1] = new Text();
		currentRow[1].data = " ";
		if (currentChannel.currentArtist != null && currentChannel.currentArtist != undefined && currentChannel.currentArtist != "")
			currentRow[1].data += (preferences.guideShowSongNames.value == "1") ? currentChannel.currentTitle : currentChannel.currentArtist;
		currentRow[1].size = fontSize;
		currentRow[1].width = (screen.width / 2) - 2;
		if (useSiriusBackstageData == 1) {
			currentRow[1].hOffset = this.frame.width - (currentRow[1].width + 2);
		} else { currentRow[1].hOffset = this.frame.width; }
		currentRow[1].vOffset = currentRow[1].height * (i + 1);
		currentRow[1].opacity = 255;
		currentRow[1].tooltip = currentChannel.currentArtist + "\n" + currentChannel.currentTitle;
		currentRow[1].onMouseUp = currentRow[0].onMouseUp;
		currentRow[1].onContextMenu = currentRow[0].onContextMenu;
		this.addSubView(currentRow[1]);

		this.rows[currentChannel.num] = currentRow;
		if (i == 0) {
			this.setScrollLineSize(currentRow[1].height);
		}
		i++;
	} while (currentChannel != this.player.siriusTuner.lastChannel);
	
	// now add the scroll bar
	this.scrollBar.width = 4;
	this.scrollBar.height = screen.height;
	this.scrollBar.hAlign = "right";
	this.scrollBar.vOffset = screen.vOffset;
	this.scrollBar.hOffset = screen.hOffset + screen.width + 1;
	this.scrollBar.window = main;
	
	this.updateColors();
}

ChannelGuideScreen.prototype.buildGuideScreenItemContextMenu = function(ch) {
	var siriusTuner = SiriusTuner.instance;
	var mi = new Array();
	mi[0] = new MenuItem();
	mi[0].title = (preferences.guideShowSongNames.value == "1") ? "Show Artist Names" : "Show Track Names";
	mi[0].onSelect = "preferences.guideShowSongNames.value = (preferences.guideShowSongNames.value * -1) + 1; Player.instance.channelGuideScreen.updateChannelData();";
	mi[1] = new MenuItem();
	mi[1].title = "-";

	if (siriusTuner.currentChannel.num != ch && siriusTuner.channels[ch] != null && siriusTuner.channels[ch].currentArtist != null && trim(siriusTuner.channels[ch].currentArtist) != "") {
		mi[2] = new MenuItem();
		mi[2].title = "Add " + siriusTuner.channels[ch].currentArtist + " to Favorites";
		mi[2].onSelect = "mnuAddArtistToAlertClicked(" + ch + ");";
		mi[3] = new MenuItem();
		mi[3].title = "Add " + siriusTuner.channels[ch].currentArtist + " - " + siriusTuner.channels[ch].currentTitle + " to Favorites";
		mi[3].onSelect = "mnuAddTitleToAlertClicked(" + ch + ");";
		mi[4] = new MenuItem();
		mi[4].title = "-";
	}
	this.rows[ch][0].contextMenuItems = mi;
	this.rows[ch][1].contextMenuItems = mi;
}

ChannelGuideScreen.prototype.setSelectedChannel = function(ch) {
	// unhighlight the currently selected channel (if any)
	if (this.selectedChannel != null) {
		this.applyShadow(this.rows[this.selectedChannel.num][0]);
		this.applyShadow(this.rows[this.selectedChannel.num][1]);
		this.rows[this.selectedChannel.num][0].color = this.skinColors("darkColor"); //"#000000";
		this.rows[this.selectedChannel.num][0].bgOpacity = 0;
		this.rows[this.selectedChannel.num][1].color = this.skinColors("darkColor"); //"#000000";
		this.rows[this.selectedChannel.num][1].bgOpacity = 0;
	}

	this.selectedChannel = ch;

	if (ch != null) {
		// highlight the channel's row and scroll to it
		if (this.rows[ch.num] != null) {
			this.rows[ch.num][0].shadow = null;
			this.rows[ch.num][0].color = this.skinColors("highlightText"); //"#FFFFFF";
			this.rows[ch.num][0].bgColor = this.skinColors("highlight"); //"#000000";
			this.rows[ch.num][0].bgOpacity = 180;

			this.rows[ch.num][1].shadow = null;
			this.rows[ch.num][1].color = this.skinColors("highlightText"); //"#FFFFFF";
			this.rows[ch.num][1].bgColor = this.skinColors("highlight"); //"#000000";
			this.rows[ch.num][1].bgOpacity = 180;
		
			// logic to prevent the scrolling from going past the beginning
			// or end of the channel list
			var scrollYNearTop = this.frame.scrollY <= 0
			var selectionNearTop = this.rows[ch.num][0].vOffset <= this.frame.height / 2;
			var scrollYNearBottom = this.frame.scrollY - screen.height <= -(this.rows[this.player.siriusTuner.lastChannel.num][0].vOffset);
			var	selectionNearBottom = this.rows[ch.num][0].vOffset >= this.rows[this.player.siriusTuner.lastChannel.num][0].vOffset - (this.frame.height / 2)
			log("scrollYNearTop: " + scrollYNearTop + ", scrollYNearBottom: " + scrollYNearBottom);
			log("selectionNearTop: " + selectionNearTop + ", selectionNearBottom: " + selectionNearBottom);
			if (!(selectionNearTop && scrollYNearTop) && !(selectionNearBottom && scrollYNearBottom)) {
				this.frame.scrollY = -this.rows[ch.num][0].vOffset + this.frame.height / 2;
			}
		}
	}
}

ChannelGuideScreen.prototype.displayChannels = function() {
	if (this.rows.length == 0) {
		this.init();
	} else {
		this.updateChannelData();
	}
	
	this.setSelectedChannel(this.player.siriusTuner.currentChannel);
}

ChannelGuideScreen.prototype.upButtonClicked = function() {
	if (this.selectedChannel != null
			&& this.selectedChannel != this.player.siriusTuner.firstChannel) {
		this.setSelectedChannel(this.selectedChannel.previousChannel);
	}
}

ChannelGuideScreen.prototype.downButtonClicked = function() {
	if (this.selectedChannel != null 
			&& this.selectedChannel != this.player.siriusTuner.lastChannel) {
		this.setSelectedChannel(this.selectedChannel.nextChannel);
	}
}

ChannelGuideScreen.prototype.guideButtonClicked = function() {
	this.player.playChannel(this.selectedChannel);
	this.player.showMainScreen();
}

ChannelGuideScreen.prototype.playButtonClicked = function() {
	log("--> ChannelGuideScreen.playButtonClicked");
	this.guideButtonClicked();
}

ChannelGuideScreen.prototype.channelClicked = function(chNum) {
	this.setSelectedChannel(this.player.siriusTuner.channels[chNum]);
	this.guideButtonClicked();
}

ChannelGuideScreen.prototype.updateChannelData = function() {
	var currentChannel = this.player.siriusTuner.firstChannel;
	var currentData = "";
	var i = 0;

	do {
		if (i > 0) currentChannel = currentChannel.nextChannel;

		this.rows[currentChannel.num][1].data = " ";
		currentData = (preferences.guideShowSongNames.value == "1") ? currentChannel.currentTitle : currentChannel.currentArtist;
		if (currentData != null && currentData != undefined && currentData != "")
			this.rows[currentChannel.num][1].data += currentData;
		this.rows[currentChannel.num][1].tooltip = currentChannel.currentArtist + "\n" + currentChannel.currentTitle;

		if (preferences.useSiriusBackstageData.value == 1) {
			this.rows[currentChannel.num][0].width = (screen.width / 2) - 3;
			this.rows[currentChannel.num][1].hOffset = this.frame.width - (this.rows[currentChannel.num][1].width + 2);
		} else {
			this.rows[currentChannel.num][0].width = (screen.width - 2);
			this.rows[currentChannel.num][1].hOffset = this.frame.width;
		}

		i++;	
	} while (currentChannel != this.player.siriusTuner.lastChannel);
}

ChannelGuideScreen.prototype.updateColors = function() {
	var headerText = this.skinColors("headerText");
	var header = this.skinColors("header");
	var highlightText = this.skinColors("highlightText");
	var highlight = this.skinColors("highlight");
	var darkColor = this.skinColors("darkColor");
	var font = this.displayFont();
	var fontStyle = this.skinColors("fontStyle");
	if (fontStyle == this.defaultColors(""))
		fontStyle = "";
	
	var alreadyApplied = false;
	alreadyApplied = (this.frame.subviews[0].font == this.displayFont());
	alreadyApplied = alreadyApplied && (this.frame.subviews[0].font == fontStyle);
	alreadyApplied = alreadyApplied && (this.frame.subviews[0].color == headerText);
	alreadyApplied = alreadyApplied && (this.frame.subviews[0].bgColor == header);
	
	if (alreadyApplied) return;
	
	for(var i = 0; i < this.frame.subviews.length; i++){
		if (String(this.frame.subviews[i]) == "[object Text]") {
			this.font = font;
			this.fontStyle = fontStyle;
			if (this.frame.subviews[i].width == screen.width) { //this is a header (category header)
				this.frame.subviews[i].color = headerText;
				this.frame.subviews[i].bgColor = header;
			} else { //this is a channel
				if (this.frame.subviews[i].bgOpacity == 180) { //this is the currentChannel (highlighted)
					this.frame.subviews[i].color = highlightText;
					this.frame.subviews[i].bgColor = highlight;
				} else {
					this.applyShadow(this.frame.subviews[i]);
					this.frame.subviews[i].color = darkColor;
				}
			}
		}
	}
	
	var scrollTrack = this.getImage("scroll_track.png");
	var scrollThumb = this.getImage("scroll_thumb.png");
	this.scrollBar.setTrackInfo(0, 0, 0, new Array(scrollTrack, scrollTrack, scrollTrack));
	this.scrollBar.setThumbInfo(0, new Array(scrollThumb, scrollThumb, scrollThumb));
	this.setScrollBar(this.scrollBar);
}