/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

include("Screen.js");

/**
 * The main screen for the Sirius player which will display channel info while
 * playing.
 */
function MainScreen(player) {
	this.player = player;
	this.init();
}

// extend the Screen object
MainScreen.prototype = new Screen();

MainScreen.prototype.init = function() {
	var fontSize = (system.platform == "windows") ? 8 : 9;

	this.albumArt = new Image();
	this.albumArt.width = preferences.useSmallAlbumArt.value == 0 ? 48 : 20;
	this.albumArt.height = this.albumArt.width;
	this.albumArt.hOffset = ((preferences.useSmallAlbumArt.value == 0) ? (screen.width / 2) - (this.albumArt.width / 2) : 6);
	this.albumArt.vOffset = ((preferences.useSmallAlbumArt.value == 0) ? (screen.height / 2) - (this.albumArt.height / 2) : this.titleText.vOffset + this.titleText.height - 12);
	this.albumArt.opacity = 0;
	this.addSubView(this.albumArt);

	this.channelText = new Text();
	this.channelText.style.textAlign = "left";
	this.channelText.size = fontSize;
	this.channelText.bgOpacity = 180;
	this.channelText.opacity = 0;
	this.channelText.data = "Channel";
	this.channelText.window = main;
	this.channelText.hOffset = 1;
	this.channelText.vOffset = this.channelText.height;
	this.addSubView(this.channelText);

	this.genreText = new Text();
	this.genreText.style.textAlign = "right";
	this.genreText.size = fontSize;
	this.genreText.bgOpacity = 180;
	this.genreText.opacity = 0;
	this.genreText.data = "GenreText";
	this.genreText.window = main;
	this.genreText.hOffset = 128;
	this.genreText.vOffset = this.channelText.vOffset;
	this.addSubView(this.genreText);
	
	this.artistText = new Text();  
	this.artistText.width = screen.width - 6;
	this.artistText.style.textAlign = "left";
	this.artistText.size = 12.5;
	this.artistText.opacity = 0;
	this.artistText.data = "Artist";
	this.artistText.scrolling = "autoLeft";
	this.artistText.window = main;
	this.artistText.hOffset = 3;
	this.artistText.vOffset = screen.height / 3;
	this.addSubView(this.artistText);
	
	this.titleText = new Text();
	this.titleText.width = this.artistText.width;
	this.titleText.style.textAlign = "left";
	this.titleText.size = 12.5;
	this.titleText.opacity = 0;
	this.titleText.data = "Title";
	this.titleText.scrolling = "autoLeft";
	this.titleText.window = main;
	this.titleText.hOffset = 3;
	this.titleText.vOffset = this.artistText.vOffset + this.artistText.height
	this.addSubView(this.titleText);
	
	this.statusText = new Text();
	this.statusText.width = screen.width - 27;
	this.statusText.style.textAlign = "left";
	this.statusText.size = fontSize;
	//this.statusText.opacity = 0;
	this.statusText.data = "Ready";
	this.statusText.scrolling = "autoLeft";
	this.statusText.window = main;
	this.statusText.hOffset = 2;
	this.statusText.vOffset = screen.height - 4;
	this.addSubView(this.statusText);
	
	this.presetBankText = new Text();
	this.presetBankText.width = 20;
	this.presetBankText.size = fontSize;
	this.presetBankText.style.fontWeight = "bold";
	this.presetBankText.opacity = 255;
	this.presetBankText.data = preferences.lastActivePresetBank.value;
	this.presetBankText.window = main;
	this.presetBankText.hOffset = screen.width - 20;
	this.presetBankText.vOffset = screen.height - 4;
	this.addSubView(this.presetBankText);
	
	this.signalStrength = new Array();
	for (var i = 0; i < 3; i++) {
		this.signalStrength[i] = new Image();
		this.signalStrength[i].src = this.getImage("signal_tick_" + (i + 1) + ".png");
		this.signalStrength[i].vAlign = "bottom";
		this.signalStrength[i].window = main;
		this.signalStrength[i].opacity = 20;
		this.signalStrength[i].hOffset = screen.width - 12 + (i * 3);
		this.signalStrength[i].vOffset = screen.height - 4;
		this.addSubView(this.signalStrength[i]);
	}

	this.alarmIcon = new Text();
	this.alarmIcon.data = String.fromCharCode(186);
	this.alarmIcon.style.fontFamily = "Wingdings";
	this.alarmIcon.style.fontWeight = "bold";
	this.alarmIcon.size = 10;
	this.alarmIcon.style.textAlign = "center";
	this.alarmIcon.window = main;
	this.alarmIcon.opacity = 40 + (preferences.enableAlarm.value * 215);
	this.alarmIcon.hOffset = screen.width - 14;
	this.alarmIcon.vOffset = screen.height - 14;
	this.addSubView(this.alarmIcon);

	this.logo = new Image();
	this.logo.src = this.getImage("logo.png");
	this.logo.window = main;
	this.logo.opacity = 255;
	this.logo.hOffset = (screen.width - this.logo.width) / 2;
	this.logo.vOffset = (screen.height - this.logo.height) / 3;
	this.addSubView(this.logo);
	
	this.updateColors();	
}

/**
 * Sets a message to appear in the status area of the screen.
 */
MainScreen.prototype.setStatusDisplay = function(text) {
	if (this.statusText.data != text) {
		if (this.statusText.opacity > 0) {
			this.statusText.fade(this.statusText.opacity, 0, 1);
		}
		this.statusText.data = text;
		this.statusText.fade(0, 255, 1);
	}
}

MainScreen.prototype.setChannelDisplay = function(ch) {
	this.channelText.data = " " + ch.num + "   " + ch.name;
	this.genreText.data = ch.category.name;
	this.channelText.width = this.channelText.width + (screen.width - 8)
			- (this.channelText.width + this.genreText.width);
	this.channelText.opacity = 255;
	this.genreText.opacity = 255;
}

MainScreen.prototype.setMainDisplay = function(ch) {
	if (ch != null && ch.currentArtist != null) {
		this.artistText.data = ch.currentArtist;
		this.titleText.data = ch.currentTitle;
	} else if (ch != null) {
		this.artistText.data = ch.name;
		this.titleText.data = "Waiting for data...";
	} else if (SiriusTuner.instance.isLoggedIn()) {
		this.artistText.data = "Invalid Channel";
		this.titleText.data = "";
	} else {
		this.artistText.data = "";
		this.titleText.data = "";
	}
	
	this.artistText.opacity = 255;
	this.titleText.opacity = 255;
}

MainScreen.prototype.updatePresetBankDisplay = function() {
	this.presetBankText.data = this.player.activePresetBank;	
}

MainScreen.prototype.updateSignalStrength = function(strength) {
	for (var i = 0; i < 3; i++) {
		if (strength >= 100 * ((i + 1) / 3)) {
			this.signalStrength[i].opacity = 255;
		} else if (strength < 100 * (i / 3) || strength == null) {
			this.signalStrength[i].opacity = 20;
		} else {
			var opacity = 255 * ((strength - (100 * (i / 3))) / (100 * (1 / 3)));
			if (opacity < 20) {
				this.signalStrength[i].opacity = 20;
			} else {
				this.signalStrength[i].opacity = opacity;
			}
		}
	}
}

MainScreen.prototype.upButtonClicked = function() {
	if(preferences.likeRadio.value == 1) {
		this.player.nextChannel(true);
	} else {
		this.player.previousChannel(true);
	}
}

MainScreen.prototype.downButtonClicked = function() {
	if(preferences.likeRadio.value == 1) {
		this.player.previousChannel(true);
	} else {
		this.player.nextChannel(true);
	}
}

MainScreen.prototype.playButtonClicked = function() {
	this.player.play();
}

MainScreen.prototype.updateChannelData = function() {
	this.setMainDisplay(this.player.siriusTuner.currentChannel);	
}

MainScreen.prototype.updateColors = function() {
	this.applyFont(this.channelText);
	this.applyFont(this.genreText);
	this.applyFont(this.artistText);
	this.applyFont(this.titleText);
	this.applyFont(this.statusText);
	this.applyFont(this.presetBankText);
	
	this.channelText.style.color = this.skinColors("highlightText"); //"#FFFFFF";
	this.channelText.style.backgroundColor = this.skinColors("highlight"); //"#000000";

	this.genreText.style.color = this.skinColors("highlightText"); //"#FFFFFF";
	this.genreText.style.backgroundColor = this.skinColors("highlight"); //"#000000";

	this.applyShadow(this.artistText);
	this.artistText.style.color = this.skinColors("darkColor");

	this.applyShadow(this.titleText);
	this.titleText.style.color = this.skinColors("darkColor");

	this.applyShadow(this.statusText);
	this.statusText.style.color = this.skinColors("darkColor");

	this.applyShadow(this.presetBankText);
	this.presetBankText.style.color = this.skinColors("darkColor");

	this.applyShadow(this.alarmIcon);
	this.alarmIcon.style.color = this.skinColors("darkColor");
	
	for (var i = 0; i < 3; i++) {
		this.signalStrength[i].src = this.getImage("signal_tick_" + (i + 1) + ".png");
	}
	this.logo.src = this.getImage("logo.png");
}

function ShrinkAlbumArt() {
	var now = animator.milliseconds;
	var t = limit(now - this.startTime, 0, this.duration);
	var percent = t / this.duration;
	var size = 48 - (28 * percent);
	var albumArt = Player.instance.mainScreen.albumArt;
	albumArt.width = size;
	albumArt.height = size;
	return (size > 20);
}

function GrowAlbumArt() {
	var now = animator.milliseconds;
	var t = limit(now - this.startTime, 0, this.duration);
	var percent = t / this.duration;
	var size = (48 / 2) + (24 * percent);
	var albumArt = Player.instance.mainScreen.albumArt;
	albumArt.width = size;
	albumArt.height = size;
	return (size < 48);	
}

function limit(num, lower, upper) {
	if (num < lower)
		return lower;
	if (num > upper)
		return upper;
	return num;
}

function floor(num) {
	var n = num + " ";
	if (n.indexOf(".") > -1) {
		n = n.substring(0, n.indexOf("."));
		return (n * 1);
	}
	return num * 1;
}

function AnimToggleAlbumArtSize() {
	var ms = Player.instance.mainScreen;
	var usaa = preferences.useSmallAlbumArt.value;
	var x = null;
	if (usaa == 1 && ms.albumArt.width == 48) {
		x = new CustomAnimation(1, ShrinkAlbumArt);
	} else if (usaa == 0 && Player.instance.mainScreen.albumArt.width == 20) {
		x = new CustomAnimation(1, GrowAlbumArt);
	}
	
	if (x != null) {
		x.duration = 500;
		var hOffset = ((usaa == 0) ? (screen.width / 2) - (48 / 2) : 6);
		var vOffset = ((usaa == 0) ? (screen.height / 2) - (48 / 2) : ms.titleText.vOffset + ms.titleText.height - 12);
		var y = new MoveAnimation(ms.albumArt, hOffset, vOffset, 500, animator.kEaseOut);
		var a = new Array(x, y);
		animator.start(a);
	}
}