/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

include("SiriusTuner.js");
include("MainScreen.js");
include("ChannelGuideScreen.js");
include("MessageScreen.js");
include("ArtistAlertScreen.js");
include("AlarmScreen.js");
include("VolumeScreen.js");

/**
 * The Main Sirius player.
 */
function Player(siriusTuner) {
	log("--> Player(siriusTuner)");
	
	// SiriusTuner instance, handles channel information and the streaming audio
	this.siriusTuner = siriusTuner;
	
	// the main screen, displays information about the current channel
	this.mainScreen = new MainScreen(this);
	
	// the channel guide screen, provides channel navigation
	this.channelGuideScreen = new ChannelGuideScreen(this);
	
	// message screen, for temporarily displaying messages on the whole screen
	this.messageScreen = new MessageScreen();
	this.messageScreenTimer = new Timer();
	this.messageScreenTimer.interval = 2;
	this.messageScreenTimer.onTimerFired = "Player.instance.hideMessage()";

	// artist alert screen, for temporarily displaying artist alerts
	this.artistAlertScreen = new ArtistAlertScreen();
	this.artistAlertScreenTimer = new Timer();
	this.artistAlertScreenTimer.interval = 60;
	this.artistAlertScreenTimer.onTimerFired = "Player.instance.hideArtistAlert()";
	
	// alarm clock screen
	this.alarmScreen = new AlarmScreen();
	this.alarmScreenTimer = new Timer();
	this.alarmScreenTimer.interval = 300;
	this.alarmScreenTimer.onTimerFired = "Player.instance.hideAlarmScreen()";
	this.alarmSnoozeTimer = new Timer();
	this.alarmSnoozeTimer.interval = preferences.snoozeTimeout.value;;
	this.alarmSnoozeTimer.onTimerFired = "checkSnoozeAlarm()";
	
	// volume screen, displays when the user changes the volume
	this.volumeScreen = new VolumeScreen();
	this.volumeScreenTimer = new Timer();
	this.volumeScreenTimer.interval = 2;
	this.volumeScreenTimer.onTimerFired = "Player.instance.hideVolumeScreen()";
	
	// holds a reference to the currently displayed screen
	// initialized to the main screen
	this.activeScreen = this.mainScreen;
	this.activeScreen.show();
	
	// the image and behaviors for the guide button
	this.guideButton = new Image(main);
	this.guideButton.src = this.mainScreen.getImage("guide_button.png");
	this.guideButton.hOffset = 225;
	this.guideButton.vOffset = 33;
	this.guideButton.tooltip = this.getGuideButtonTooltip();
	this.guideButton.onMouseDown = "Player.instance.guideButtonDown()";
	this.guideButton.onMouseUp = "Player.instance.guideButtonUp()";
		
	// the up and down buttons
	this.upButton = new Image(main);
	this.upButton.src = this.mainScreen.getImage("chup_button.png");
	this.upButton.hOffset = 218;
	this.upButton.vOffset = 16;
	this.upButton.tooltip = this.getUpButtonTooltip();
	this.upButton.onMouseDown = "Player.instance.upButtonDown()";
	this.upButton.onMouseUp = "Player.instance.upButtonUp()";
	
	this.downButton = new Image(main);
	this.downButton.src = this.mainScreen.getImage("chdown_button.png");
	this.downButton.hOffset = 216;
	this.downButton.vOffset = 54;
	this.downButton.tooltip = this.getDownButtonTooltip();
	this.downButton.onMouseDown = "Player.instance.downButtonDown()";
	this.downButton.onMouseUp = "Player.instance.downButtonUp()";
	
	// the stop and play buttons
	this.stopButton = new Image(main);
	this.stopButton.src = this.mainScreen.getImage("stop_button.png");
	this.stopButton.hOffset = 201;
	this.stopButton.vOffset = 25;
	this.stopButton.tooltip = "Stop [Left Arrow]";
	this.stopButton.onMouseDown = "Player.instance.stopButtonDown()";
	this.stopButton.onMouseUp = "Player.instance.stopButtonUp()";
	
	this.playButton = new Image(main);
	this.playButton.src = this.mainScreen.getImage("play_button.png");
	this.playButton.hOffset = 245;
	this.playButton.vOffset = 25;
	this.playButton.tooltip = "Play [Right Arrow]";
	this.playButton.onMouseDown = "Player.instance.playButtonDown()";
	this.playButton.onMouseUp = "Player.instance.playButtonUp()";
	
	// the volume buttons
	this.volumeUpButton = new Image(main);
	this.volumeUpButton.src = this.mainScreen.getImage("volume_up.png");
	this.volumeUpButton.hOffset = 192;
	this.volumeUpButton.vOffset = 19;
	this.volumeUpButton.tooltip = "Volume up [+]";
	this.volumeUpButton.onMouseDown = "Player.instance.volumeUpButtonDown()";
	this.volumeUpButton.onMouseUp = "Player.instance.volumeUpButtonUp()";
	
	this.volumeDownButton = new Image(main);
	this.volumeDownButton.src = this.mainScreen.getImage("volume_down.png");
	this.volumeDownButton.hOffset = 189;
	this.volumeDownButton.vOffset = 37;
	this.volumeDownButton.tooltip = "Volume down [-]";
	this.volumeDownButton.onMouseDown = "Player.instance.volumeDownButtonDown()";
	this.volumeDownButton.onMouseUp = "Player.instance.volumeDownButtonUp()";
	
	this.muteButton = new Image(main);
	this.muteButton.src = this.mainScreen.getImage("mute_button.png");
	this.muteButton.hOffset = 189;
	this.muteButton.vOffset = 52;
	this.muteButton.tooltip = "Mute [m]";
	this.muteButton.onMouseDown = "Player.instance.muteButtonDown()";
	this.muteButton.onMouseUp = "Player.instance.muteButtonUp()";
	
	// stores the active preset bank: A, B, C or direct
	this.activePresetBank = preferences.lastActivePresetBank.value;
	
	// preset bank button
	this.presetBankButton = new Image(main);
	this.presetBankButton.src = this.mainScreen.getImage("preset_button.png");
	this.presetBankButton.hOffset = 176;
	this.presetBankButton.vOffset = 97;
	this.presetBankButton.tooltip = "Switch preset bank [p]\nUse [a], [b], [c], and [#] to switch directly";
	this.presetBankButton.onMouseDown = "Player.instance.presetBankButtonDown()";
	this.presetBankButton.onMouseUp = "Player.instance.presetBankButtonUp()";
	this.presetBankButton.onContextMenu = "Player.instance.buildPresetBankButtonContextMenu()";
	
	// power button
	this.powerButton = new Image(main);
	this.powerButton.src = this.mainScreen.getImage("power_button.png");
	this.powerButton.hOffset = 225;
	this.powerButton.vOffset = 103;
	this.powerButton.tooltip = "Close Sirius Tuner [x]";
	this.powerButton.onMouseDown = "Player.instance.powerButtonDown()";
	this.powerButton.onMouseUp = "Player.instance.powerButtonUp()";
	
	// initialize the number buttons
	this.numberButtons;
	this.initNumberButtons();

	// timer for maintaining a regular refresh of what's currently playing
	// on all channels
	this.refreshWhatsOnTimer = new Timer();
	this.refreshWhatsOnTimer.interval = preferences.refreshWhatsOnInterval.value;
	this.refreshWhatsOnTimer.onTimerFired = "Player.instance.refreshWhatsOn()";
	this.refreshWhatsOnCount = 0;
		
	// timer for allowing the user to channel surf in the main screen.  When
	// channel up/down is clicked, the screen will immediately display the
	// channel info, but it will wait for this timer to fire before opening
	// the audio stream
	this.changeChannelTimer = new Timer();
	this.changeChannelTimer.interval = 2;
	this.changeChannelTimer.onTimerFired = "Player.instance.changeChannelTimerFired()";
	
	// holds the preset button that was just pressed, so that the timer that
	// fires after 2 seconds will know which preset button was pressed.
	this.presetButtonDown;
	this.presetButtonTimer = new Timer();
	this.presetButtonTimer.interval = 1.5;
	this.presetButtonTimer.onTimerFired = "Player.instance.presetButtonHeldDown(Player.instance.presetButtonDown)";

	// buffer for channel number direct entry
	this.directEntryBuffer = "";
	this.directEntryTimer = new Timer();
	this.directEntryTimer.interval = 3;
	this.directEntryTimer.onTimerFired = "Player.instance.handleDirectEntry()";
	
	// timer for updating the signal strength indicator on the main screen
	this.signalStrengthTimer = new Timer();
	this.signalStrengthTimer.interval = 1;
	this.signalStrengthTimer.ticking = true;
	this.signalStrengthTimer.onTimerFired = "Player.instance.mainScreen.updateSignalStrength(SiriusTuner.instance.getReceptionQuality())";
	
	this.ReconnectTimer = new Timer();
	this.ReconnectTimer.interval = 2;
	this.ReconnectTimer.ticking = false;
	this.ReconnectTimer.onTimerFired = "Player.instance.AttemptReconnect()";
	
	this.muted = false;
	
	this.hotKeys;
	this.initHotkeys();
}

Player.prototype.initNumberButtons = function() {
	this.numberButtons = new Array();
	for (var i = 0; i <= 9; i++) {
		var num = ((i + 1) == 10)? 0 : (i + 1);
		this.numberButtons[num] = new Image();
		this.numberButtons[num].src = this.mainScreen.getImage(num + "_button.png");
		this.numberButtons[num].hOffset = 20 + (i * 15);
		this.numberButtons[num].vOffset = 99;
		this.numberButtons[num].window = main;
		this.numberButtons[num].onMouseDown = "Player.instance.presetButtonPressed(" + num + ")";
		this.numberButtons[num].onMouseUp = "Player.instance.presetButtonReleased(" + num + ")";
		this.numberButtons[num].onContextMenu = "Player.instance.buildPresetButtonContextMenu(" + num + ")";
	}
	this.updateNumberButtonTooltips()
}

Player.prototype.updateNumberButtonTooltips = function() {
	for (var i = 0; i <= 9; i++) {
		var num = i + 1;
		var index = (num == 10)? 0 : num;		
		if (this.activePresetBank != "#") {
			this.numberButtons[index].tooltip = this.activePresetBank + num + ": ";
			var channel = this.getPresetChannel(num);
			if (channel != null && channel != "") {
				this.numberButtons[index].tooltip += channel;
				if (this.siriusTuner.channels != null) {
					channel = this.siriusTuner.channels[channel];
					if (channel != null) {
						this.numberButtons[index].tooltip += " - " + channel.name;
						if (preferences.useSiriusBackstageData.value == 1) { this.numberButtons[index].tooltip += "\n" + channel.currentArtist + "\n" + channel.currentTitle; }
					}
				}
			} else {
				this.numberButtons[index].tooltip += "not set";
			}
		} else {
			this.numberButtons[index].tooltip = "#" + index;
		}
	}
}

/**
 * Logs in to Sirius Radio Online using the username and password supplied.
 */
Player.prototype.login = function() {
	// start the login process
	this.mainScreen.setStatusDisplay("Logging in to Sirius Satellite Radio");
	try {
		// attempt to log in to Sirius Radio Online
		this.siriusTuner.login();
	} catch (e) {
		this.mainScreen.setStatusDisplay(e);
	}
	
	// make sure the login was successful
	if (this.siriusTuner.isLoggedIn()) {
		this.mainScreen.setStatusDisplay("Logged in, retrieving channels...");
		
		// initialize everything
		this.siriusTuner.getWhatsOnSources();
		this.siriusTuner.loadChannelData();
		// load channel data and start the channel data refresh timer if enabled
		if (preferences.useSiriusBackstageData.value == 1) {
			this.siriusTuner.loadWhatsOn();
			this.refreshWhatsOnTimer.ticking = true;
		}
		this.mainScreen.setStatusDisplay("Initializing Windows Media Player...");
		if (system.platform == "windows" && this.siriusTuner.wmp == null) { this.siriusTuner.initPlayer(); }
		this.mainScreen.logo.opacity = 0;
	} else {
		resetCachedLogin();
	}
}

/**
 * Plays the specified channel.  The channel argument can either be a valid
 * channel number or an SiriusChannel instance.  If the wait parameter is
 * false or not passed, the channel will begin playing immediately.  If the wait
 * parameter is true, the change channel timer will trigger the channel
 * to start playing.  This enables the user to quickly skip past channels when
 * using the channel up/down buttons in the main screen as it prevents the
 * widget from immediately retrieving the stream and trying to play it.
 */
Player.prototype.playChannel = function(ch, wait) { //playChannel(ch, [wait = false])
	if (this.siriusTuner.isLoggedIn()) {
		try {
			// load the channel
			log("Trying setCurrentChannel (playChannel): " + ch);
			ch = this.siriusTuner.setCurrentChannel(ch);

			setMessengerStatus(ch);
			
			// update the main screen with the channel info
			this.mainScreen.setChannelDisplay(this.siriusTuner.currentChannel);
			this.mainScreen.setMainDisplay(this.siriusTuner.currentChannel);
			
			if (!wait) {
				// play the channel immediately
				eval(this.changeChannelTimer.onTimerFired);
			} else {
				// play the channel when the timer fires
				this.changeChannelTimer.reset();
				this.changeChannelTimer.ticking = true;
			}
			
			// set this channel as the last channel played
			preferences.lastChannel.value = this.siriusTuner.currentChannel.num;
		} catch (e) {
			this.mainScreen.setStatusDisplay(e);
		}
	}
}

Player.prototype.setSelectedChannel = function(ch) {
	if (this.siriusTuner.isLoggedIn()) {
		try {
			// load the channel
			log("Trying setSelectedChannel: " + ch);
			this.siriusTuner.setCurrentChannel(ch);
			
			// update the main screen with the channel info
			this.mainScreen.setChannelDisplay(this.siriusTuner.currentChannel);
			this.mainScreen.setMainDisplay(this.siriusTuner.currentChannel);
			
			// set this channel as the last channel played
			preferences.lastChannel.value = this.siriusTuner.currentChannel.num;
		} catch (e) {
			this.mainScreen.setStatusDisplay(e);
		}
	}
}

Player.prototype.play = function() {
	this.siriusTuner.play();	
}

Player.prototype.stop = function() {
	this.siriusTuner.stop();
}

/**
 * Changes to the next channel in the lineup.
 */
Player.prototype.nextChannel = function(bynum) {	
	if (this.siriusTuner.currentChannel != null) {
		if (bynum) {
			this.playChannel(this.siriusTuner.currentChannel.nextChannelByNum(), true);
		} else {
			this.playChannel(this.siriusTuner.currentChannel.nextChannel, true);
		}
	}
}

/**
 * Changes to the previous channel in the lineup.
 */
Player.prototype.previousChannel = function(bynum) {	//deprecated
	if (this.siriusTuner.currentChannel != null) {
		if (bynum) {
			this.playChannel(this.siriusTuner.currentChannel.previousChannelByNum(), true);
		} else {
			this.playChannel(this.siriusTuner.currentChannel.previousChannel, true);
		}
	}
}

/**
 * Refreshes channel data.
 */ 
Player.prototype.refreshWhatsOn = function() {
	if ((preferences.useSiriusBackstageData.value == 1) && this.siriusTuner.isLoggedIn())
		if (this.refreshWhatsOnCount > 4) {
			this.siriusTuner.getWhatsOnSources();
			this.refreshWhatsOnCount = 0;
		} else { this.refreshWhatsOnCount++; }
		this.siriusTuner.loadWhatsOn();
}

/**
 * Signals the active screen to update channel data.
 */
Player.prototype.updateScreen = function() {
	this.activeScreen.updateChannelData();
	this.updateNumberButtonTooltips();
}

/**
 * Called by the channel change timer to play the selected channel.
 */
Player.prototype.changeChannelTimerFired = function() {
	this.changeChannelTimer.ticking = false;
	
	/* *** don't automatically play the selected channel if the player is stopped ***
	 Was programmed according to the line above, but now I can't understand the logic
	 behind why I programmed it that way. As a result the following line has been
	 commented out.
	 if (!this.siriusTuner.isStopped()) */
		this.play();
}

/**
 * Show the channel guide as the active screen.
 */
Player.prototype.showGuide = function() {
	if (this,siriusTuner.isLoggedIn()) {
		if (this.siriusTuner.currentChannel == null)
			this.siriusTuner.nextChannel();
			
		this.activeScreen.hide();
		this.channelGuideScreen.displayChannels();
		this.activeScreen = this.channelGuideScreen;
		this.activeScreen.show();
	}
}

/**
 * Show the main screen as the active screen.
 */
Player.prototype.showMainScreen = function() {
	this.activeScreen.hide();
	this.activeScreen = this.mainScreen;
	this.activeScreen.show();
}

/**
 * Handle the stop button.
 */
Player.prototype.stopButtonClicked = function() {
	this.stop();
}

/**
 * Handle the play button.
 */
Player.prototype.playButtonClicked = function() {
	this.activeScreen.playButtonClicked();
}

/**
 * Handle the up button.
 */
Player.prototype.upButtonDown = function() {
	if (this.upButton.src.search("_inset") == -1)
		this.upButton.src = this.mainScreen.getImage("chup_button_inset.png");
}

Player.prototype.upButtonUp = function() {
	this.upButton.src = this.mainScreen.getImage("chup_button.png");
	this.upButtonClicked();	
}
 
Player.prototype.upButtonClicked = function() {
	this.activeScreen.upButtonClicked();
}

Player.prototype.getUpButtonTooltip = function() {
	if (this.activeScreen instanceof ChannelGuideScreen)
		return "Scroll up [Up Arrow]";
	else
		return "Next Channel [Up Arrow]";
}

/**
 * Handle the down button.
 */
Player.prototype.downButtonDown = function() {
	if (this.downButton.src.search("_inset") == -1)
		this.downButton.src = this.mainScreen.getImage("chdown_button_inset.png");
}

Player.prototype.downButtonUp = function() {
	this.downButton.src = this.mainScreen.getImage("chdown_button.png");
	this.downButtonClicked();	
}

Player.prototype.downButtonClicked = function() {
	this.activeScreen.downButtonClicked();
}

Player.prototype.getDownButtonTooltip = function() {
	if (this.activeScreen instanceof ChannelGuideScreen)
		return "Scroll down [Down Arrow]";
	else
		return "Previous Channel [Down Arrow]";
}

Player.prototype.stopButtonDown = function() {
	if (this.stopButton.src.search("_inset") == -1)
		this.stopButton.src = this.mainScreen.getImage("stop_button_inset.png");
}

Player.prototype.stopButtonUp = function() {
	this.stopButton.src = this.mainScreen.getImage("stop_button.png");
	this.stopButtonClicked();
}

Player.prototype.playButtonDown = function() {
	if (this.playButton.src.search("_inset") == -1)
		this.playButton.src = this.mainScreen.getImage("play_button_inset.png");
}

Player.prototype.playButtonUp = function() {
	this.playButton.src = this.mainScreen.getImage("play_button.png");
	this.playButtonClicked();
}

Player.prototype.getGuideButtonTooltip = function() {
	if (this.activeScreen instanceof ChannelGuideScreen || this.activeScreen instanceof ArtistAlertScreen)
		return "Play the selected channel [Enter]";
	else
		return "View the channel guide [g]";
}

Player.prototype.guideButtonDown = function() {
	if (this.guideButton.src.search("_inset") == -1)
		this.guideButton.src = this.mainScreen.getImage("guide_button_inset.png");
}

Player.prototype.guideButtonUp = function() {
	this.guideButton.src = this.mainScreen.getImage("guide_button.png");
	this.guideButtonClicked();
}

/**
 * Handle the guide button.
 */
Player.prototype.guideButtonClicked = function() {
	if (this.activeScreen instanceof ChannelGuideScreen || this.activeScreen instanceof ArtistAlertScreen || this.activeScreen instanceof AlarmScreen)
		this.activeScreen.guideButtonClicked();
	else
		this.showGuide();

	this.guideButton.tooltip = this.getGuideButtonTooltip();
	this.downButton.tooltip = this.getDownButtonTooltip();
	this.upButton.tooltip = this.getUpButtonTooltip();                                           
}

Player.prototype.presetBankButtonDown = function() {
	if (this.presetBankButton.src.search("_inset") == -1)
		this.presetBankButton.src = this.mainScreen.getImage("preset_button_inset.png");
}

Player.prototype.presetBankButtonUp = function() {
	this.presetBankButton.src = this.mainScreen.getImage("preset_button.png");
	this.switchPresetBank();
}

Player.prototype.presetButtonPressed = function(num) {
	if (this.numberButtons[num].src.search("_inset") == -1) {
		this.numberButtons[num].src = this.mainScreen.getImage(num + "_button_inset.png");
	}
	if (num == 0 && this.activePresetBank != "#") num = 10;
	log("preset button pressed " + num);
	if (this.activePresetBank != "#") {
		this.presetButtonDown = num;
		this.presetButtonTimer.ticking = true;
	} else {
		this.directEntryTimer.ticking = false;
	}
}

Player.prototype.presetButtonReleased = function(num) {
	this.numberButtons[num].src = this.mainScreen.getImage(num + "_button.png");
	if (num == 0 && this.activePresetBank != "#") num = 10;
	log("preset button released " + num);
	
	if (this.activePresetBank != "#") {
		this.presetButtonTimer.ticking = false;
		this.presetButtonDown = null;
		
		// switch to the preset channel, if any
		var presetChannel = this.getPresetChannel(num);
		if (presetChannel != null && presetChannel != "")
			presetChannel = this.siriusTuner.channels[presetChannel];

		if (presetChannel != null && presetChannel != "")
			this.playChannel(presetChannel);
		else
			this.mainScreen.setStatusDisplay("Preset " + this.activePresetBank + num + " not set");
	} else {
		this.directEntryBuffer += num;
		this.displayMessage(this.directEntryBuffer, 3);
		if (this.directEntryBuffer.length == 3)
			this.handleDirectEntry();
		else
			this.directEntryTimer.ticking = true;
	}
}

Player.prototype.buildPresetBankButtonContextMenu = function() {
	var items = new Array();
	
	items[0] = new MenuItem();
	items[0].title = "Preset Bank A";
	items[0].checked = (this.activePresetBank == "A");
	items[0].onSelect = "Player.instance.switchPresetBank('A')";
	
	items[1] = new MenuItem();
	items[1].title = "Preset Bank B";
	items[1].checked = (this.activePresetBank == "B");
	items[1].onSelect = "Player.instance.switchPresetBank('B')";
	
	items[2] = new MenuItem();
	items[2].title = "Preset Bank C";
	items[2].checked = (this.activePresetBank == "C");
	items[2].onSelect = "Player.instance.switchPresetBank('C')";
	
	items[3] = new MenuItem();
	items[3].title = "Direct Entry";
	items[3].checked = (this.activePresetBank == "#");
	items[3].onSelect = "Player.instance.switchPresetBank('#')";
	
	items[4] = new MenuItem();
	items[4].title = "-";
	
	this.presetBankButton.contextMenuItems = items;
}

Player.prototype.getPresetChannel = function(num) {
	var preset;
	eval("preset = preferences.preset" + this.activePresetBank + num);
	if (preset != null)
		return preset.value;
	else
		return null;
}

Player.prototype.setPresetChannel = function(num) {
	var preset;
	eval("preset = preferences.preset" + this.activePresetBank + num);
	if (preset != null)
		preset.value = this.siriusTuner.currentChannel.num;

	this.updateNumberButtonTooltips();
}

Player.prototype.presetButtonHeldDown = function(num) {
	if (num == 0 && this.activePresetBank != "#") num = 10;
	log("preset button held down " + num);
	this.presetButtonTimer.ticking = false;
	this.presetButtonDown = null;
	
	// store the preset
	this.setPresetChannel(num);
	this.displayMessage("Preset " + this.activePresetBank + num + " set");
}

Player.prototype.buildPresetButtonContextMenu = function(num) {
	if (this.activePresetBank != "#") {
		var items = new Array();
		
		items[0] = new MenuItem();
		items[0].title = "Set preset " + this.activePresetBank + num + " to " + this.siriusTuner.currentChannel.name;
		items[0].enabled = (this.getPresetChannel(num) != this.siriusTuner.currentChannel.num);
		items[0].onSelect = "Player.instance.setPresetChannel(" + num + ")";
		
		items[1] = new MenuItem();
		items[1].title = "-";
		
		this.numberButtons[num].contextMenuItems = items;
	}
}

Player.prototype.handleDirectEntry = function() {
	this.directEntryTimer.ticking = false;
	this.hideMessage();
	this.playChannel(this.directEntryBuffer);
	this.directEntryBuffer = "";
}

Player.prototype.switchPresetBank = function(bank) {
	log("Switching preset bank from " + this.activePresetBank);
	if (bank == null) {
		switch (this.activePresetBank) {
			case "A":
				this.activePresetBank = "B";
				break;
			case "B":
				this.activePresetBank = "C";
				break;
			case "C":
				this.activePresetBank = "#";
				break;
			case "#":
				this.activePresetBank = "A";
				break;
		}
	} else {
		this.activePresetBank = bank.toUpperCase();
	}
	log("to " + this.activePresetBank);
	this.updateNumberButtonTooltips();
	preferences.lastActivePresetBank.value = this.activePresetBank;
	this.mainScreen.updatePresetBankDisplay();
	log("Preset bank " + this.activePresetBank + " selected");
}

Player.prototype.displayMessage = function(msg, howLong) {
	this.messageScreen.setMessage(msg);
	this.activeScreen.hide();
	this.messageScreen.show();
	if (howLong > 0)
		this.messageScreenTimer.interval = howLong;
	this.messageScreenTimer.ticking = true;
}

Player.prototype.hideMessage = function() {
	this.messageScreenTimer.ticking = false;
	this.messageScreen.hide();
	this.activeScreen.show();
}

Player.prototype.displayArtistAlert = function(ch) {
	if (!(Player.instance.activeScreen instanceof AlarmScreen)) {
		this.artistAlertScreen.setMessage(ch);
		this.activeScreen.hide();
		this.artistAlertScreen.show();
		this.activeScreen = this.artistAlertScreen;
		this.artistAlertScreenTimer.ticking = true;
		this.artistAlertScreenTimer.reset();
	}
}

Player.prototype.hideArtistAlert = function() {
	this.artistAlertScreenTimer.ticking = false;
	this.artistAlertScreen.hide();
	this.activeScreen = this.mainScreen;
	this.activeScreen.show();
}

Player.prototype.displayAlarmScreen = function() {
	this.alarmScreen.setMessage(preferences.hour.value + ":" + preferences.min.value);
	this.activeScreen.hide();
	this.activeScreen = this.alarmScreen;
	this.activeScreen.show();
	this.alarmScreenTimer.ticking = true;
	this.alarmScreenTimer.reset();
}

Player.prototype.hideAlarmScreen = function() {
	this.alarmScreenTimer.ticking = false;
	this.alarmScreen.hide();
	this.activeScreen = this.mainScreen;
	this.activeScreen.show();
}

Player.prototype.volumeUpButtonDown = function() {
	if (this.volumeUpButton.src.search("_inset") == -1)
		this.volumeUpButton.src = this.mainScreen.getImage("volume_up_inset.png");
}

Player.prototype.volumeUpButtonUp = function() {
	this.volumeUpButton.src = this.mainScreen.getImage("volume_up.png");
	this.volumeUpButtonClicked();
}

Player.prototype.volumeUpButtonClicked = function() {
	this.displayVolumeScreen();
	this.siriusTuner.setVolume(this.siriusTuner.getVolume() + 5);
	this.volumeScreen.updateVolume(this.siriusTuner.getVolume());
}

Player.prototype.volumeDownButtonDown = function() {
	if (this.volumeDownButton.src.search("_inset") == -1)
		this.volumeDownButton.src = this.mainScreen.getImage("volume_down_inset.png");
}

Player.prototype.volumeDownButtonUp = function() {
	this.volumeDownButton.src = this.mainScreen.getImage("volume_down.png");
	this.volumeDownButtonClicked();
}

Player.prototype.volumeDownButtonClicked = function() {
	this.displayVolumeScreen();
	this.siriusTuner.setVolume(this.siriusTuner.getVolume() - 5);
	this.volumeScreen.updateVolume(this.siriusTuner.getVolume());
}

Player.prototype.muteButtonDown = function() {
	if (this.muteButton.src.search("_inset") == -1)
		this.muteButton.src = this.mainScreen.getImage("mute_button_inset.png");
}

Player.prototype.muteButtonUp = function() {
	this.muteButtonClicked();
	if (!this.muted)
		this.muteButton.src = this.mainScreen.getImage("mute_button.png");
}

Player.prototype.muteButtonClicked = function() {
	this.siriusTuner.mute();
}

Player.prototype.powerButtonDown = function() {
	if (this.powerButton.src.search("_inset") == -1)
		this.powerButton.src = this.mainScreen.getImage("power_button_inset.png");
}

Player.prototype.powerButtonUp = function() {
	this.powerButton.src = this.mainScreen.getImage("power_button.png");
	closeWidget();
}

Player.prototype.displayVolumeScreen = function() {
	this.volumeScreenTimer.ticking = false;
	this.activeScreen.hide();
	this.volumeScreen.show();
	this.volumeScreenTimer.ticking = true;
}

Player.prototype.hideVolumeScreen = function() {
	this.volumeScreenTimer.ticking = false;
	this.volumeScreen.hide();
	this.activeScreen.show();
}

Player.prototype.initHotkeys = function() {
	if (this.hotKeys == null)
		this.hotKeys = new Array();

	if (this.hotKeys["focusWidgetHotKey"] == null)
		this.hotKeys["focusWidgetHotKey"] = new HotKey();
	this.hotKeys["focusWidgetHotKey"].modifier = preferences.focusWidgetHotKey.value.substring(0, preferences.focusWidgetHotKey.value.lastIndexOf("+"));
	this.hotKeys["focusWidgetHotKey"].key      = preferences.focusWidgetHotKey.value.substring(preferences.focusWidgetHotKey.value.lastIndexOf("+") + 1, preferences.focusWidgetHotKey.value.length);
	this.hotKeys["focusWidgetHotKey"].onKeyDown = "focusWidget()";

	for (var num = 0; num <= 9; num++) {
		var val = eval("preferences.preset" + num + "HotKey.value");
		if (val.length > 0) {
			if (this.hotKeys["preset" + num + "HotKey"] == null)
				this.hotKeys["preset" + num + "HotKey"] = new HotKey();
			this.hotKeys["preset" + num + "HotKey"].modifier = val.substring(0, val.lastIndexOf("+"));
			this.hotKeys["preset" + num + "HotKey"].key = val.substring(val.lastIndexOf("+") + 1, val.length);
			this.hotKeys["preset" + num + "HotKey"].onKeyDown = "if (Player.instance.activePresetBank != \"#\") { Player.instance.presetButtonReleased(" + num + "); }";
		}
	}
}

// singleton instance of the player object
Player.instance = new Player(SiriusTuner.instance);

Player.prototype.AttemptReconnect = function() {
	if (system.platform == "windows" && player.siriusTuner.wmp.playState == 10) {
		if (player.siriusTuner.channelPlaying != null) {
			player.siriusTuner.play();
		}
	}
}

function wmpPlayStateChange(newstate) {
	Player.instance.mainScreen.albumArt.opacity = 0;
	
	var str = "Unknown";
	switch (newstate) {
		case 0: str = "Undefined"; break;
		case 1: str = "Stopped"; break;
		case 2: str = "Paused"; break;
		case 3: str = "Playing"; break;
		case 4: str = "ScanForward"; break;
		case 5: str = "ScanReverse"; break;
		case 6: str = "Buffering"; break;
		case 7: str = "Waiting"; break;
		case 8: str = "MediaEnded"; break;
		case 9: str = "Transitioning"; break;
		case 10: str = "Ready"; break;
		case 11: str = "Reconnecting"; break;
	}

	log("PlayStateChange: " + newstate + " (" + str + ")");
	if (newstate == 10) {
		player.ReconnectTimer.ticking = true;
	}
}

// listens for player status changes from Windows Media player and updates
// the main screen status display accordingly
function wmpStatusChange() {
	if (system.platform == "windows" && player != null)
		if (player.siriusTuner.wmp.status != "" && player.siriusTuner.wmp.status != null) {		
			var status = player.siriusTuner.wmp.status;
			status = status.replace(/ 'reflector:[0-9]{5}'/, '');
			log("Status: " + status);
			player.mainScreen.setStatusDisplay(status);
		}
}

function wmpScriptCommand(type, param) {
	//get start index of title and artist in caption param
	var titleIndex = param.indexOf("TITLE");
	var artistIndex = param.indexOf("ARTIST");
	
	//parse caption param to extract title and artist info
	if (titleIndex != -1) {
		var titleAdvance = 6;
		var artistAdvance = 7;
		var end = param.length;
		var titleStart = titleIndex + titleAdvance;
		var artistStart = artistIndex + artistAdvance;
		var title = param.slice(titleStart, (artistIndex - 1));
		var artist = param.slice(artistStart, end);
		
		player.siriusTuner.currentChannel.currentArtist = artist;
		player.siriusTuner.currentChannel.currentTitle = title;
		player.updateScreen();

		setAlbumArt(artist, title);
		setMessengerStatus(player.siriusTuner.currentChannel);
	}
	
	return;
}

var lastRequest = "";

function setAlbumArt(artist, title) {
	if (preferences.enableAlbumArt.value == 0) return;

	var img = Player.instance.mainScreen.albumArt;
	var keyword = artist + " " + title;
	
	if (lastRequest == keyword) return;
	lastRequest = keyword;
	log("Getting Album Art: " + keyword);
	
	var doc = XMLDOM.parse(filesystem.readFile("albumart.xml"));
	var nl = doc.getElementsByName("keyword");
	var tn = doc.createTextNode(keyword);
	nl.item(0).appendChild(tn);
	var xml = doc.toXML().replace(/>\s+/g, ">").replace(/\s+</g, "<");

	var request = new URL();
	request.location = "http://soap.amazon.com/onca/soap3";
	request.postData = xml;
	request.setRequestHeader("SOAPAction", "http://soap.amazon.com")
	request.setRequestHeader("Content-Type", "text/xml")

	try {	
		request.fetchAsync(receiveAlbumArtUrl);
	} catch (e) { img.opacity = 0; }
}

function receiveAlbumArtUrl(request) {
	if (request.response == 200) {
		try {
			var img = Player.instance.mainScreen.albumArt;
			var doc = XMLDOM.parse(request.responseData);
			var nl = doc.getElementsByName("ImageUrlSmall");
			
			if (nl.length > 0) {
				img.src = nl.item(0).firstChild.data;
				img.opacity = 255 * 0.8;
				return;
			}
		} catch (e) { }
	}
	img.opacity = 0;
}