/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

/**
 * Defines a generic screen for the Sirius player, specific types of screens
 * can extend from this.
 */
function Screen() {
	this.frame = new Frame();
	this.frame.width = screen.width;
	this.frame.height = screen.height;
	this.frame.hOffset = screen.hOffset;
	this.frame.vOffset = screen.vOffset;
	this.frame.window = main;
	this.frame.visible = false;
}

/**
 * Pass-through to the Frame.addSubView() method
 */
Screen.prototype.addSubView = function(element) {
	this.frame.addSubview(element);
}

Screen.prototype.setScrollBar = function(scrollBar) {
	this.frame.vScrollBar = scrollBar;
}

Screen.prototype.setScrollLineSize = function(size) {
	this.frame.vLineSize = size;
}

Screen.prototype.show = function() {
	this.frame.hOffset = screen.hOffset;
	this.frame.vOffset = screen.vOffset;
	if (this.frame.vScrollBar != null) {
		this.frame.vScrollBar.hOffset = screen.hOffset + 151;
		this.frame.vScrollBar.vOffset = screen.vOffset;
	}
	
	this.updateColors();
	this.frame.visible = true;
	if (this.frame.vScrollBar != null) {
		this.frame.vScrollBar.visible = true;
	}
}

Screen.prototype.hide = function() {
	this.frame.visible = false;
	if (this.frame.vScrollBar != null) {
		this.frame.vScrollBar.visible = false;
	}
}

Screen.prototype.useShadow = function() {
	var wdf = system.widgetDataFolder;
	var skinName = preferences.currentSkin.value;
	var skinPath = wdf + "/skins";
	var infoPath = skinPath + "/" + skinName + "/info.xml";
	var ret = false;
	
	if (filesystem.itemExists(infoPath)) {
		var infoText = filesystem.readFile(infoPath);
		var doc = XMLDOM.parse(infoText);
		ret = doc.evaluate("string(siriusTunerSkin/useShadow)");
	}
	return stringToBoolean(ret);
}

Screen.prototype.skinColors = function(key) {
	var wdf = system.widgetDataFolder;
	var skinName = preferences.currentSkin.value;
	var skinPath = wdf + "/skins";
	var infoPath = skinPath + "/" + skinName + "/info.xml";
	var ret = this.defaultColors(key);

	if (filesystem.itemExists(infoPath)) {
		var infoText = filesystem.readFile(infoPath);
		var doc = XMLDOM.parse(infoText);
		ret = doc.evaluate("string(siriusTunerSkin/" + key + ")");
		if (ret.length == 0)
			ret = this.defaultColors(key);
	}
	return ret;
}

Screen.prototype.defaultColors = function(key) {
	switch (key.toLowerCase()) {
		case "lightcolor": return "#FFFFFF"; break;
		case "darkcolor": return "#000000"; break;
		case "highlight": return "#000000"; break;
		case "highlighttext": return "#FFFFFF"; break;
		case "header": return "#CCCCCC"; break;
		case "headertext": return "#000000"; break;
		case "shadowcolor": return "#000000"; break;
		default: return "#FF0000"; //if you see red, then something is spelled wrong somewhere, unless your skin calls for red.
	}
}

Screen.prototype.applyFont = function(item) {
	var fontStyle = this.skinColors("fontStyle");
	if (fontStyle == this.defaultColors(""))
		fontStyle = "";
	item.style.fontFamily = this.displayFont();
	if (fontStyle.length > 0) item.style.fontStyle = fontStyle;
}

Screen.prototype.displayFont = function() {
	//cheat by using the skinColors function...
	var ret = this.skinColors("font");
	if (ret == this.defaultColors(""))
		ret = preferences.displayFont.value;
	return ret;
}

Screen.prototype.applyShadow = function(item) {
	if (this.useShadow()) {
		var s = new Shadow();
		s.color = this.skinColors("shadowColor");
		item.shadow = s;
	} else {
		item.shadow = null;
	}
}

Screen.prototype.getImage = function(img) {
	var wdf = system.widgetDataFolder;
	var skinPath = wdf + "/skins";
	var currentSkin = skinPath + "/" + preferences.currentSkin.value;
	var reqImagePath = currentSkin + "/" + img;
	
	if (filesystem.itemExists(reqImagePath)) {
		return reqImagePath;
	} else {
		return "resources/images/" + img;
	}
}