/*
Sirius Tuner Widget
by Tanner Jepsen (neoscrib@gmail.com)

Not affiliated with or supported by Sirius Radio Online
Copyright (c) 2006 Tanner Jepsen.  All rights reserved.
*/

/**
 * The Sirius Tuner.
 */

var servlet_url = preferences.servlet.value;
var urls = new Array();
var previousWhatsOnData = new Array();
previousWhatsOnData[0] = 0;
previousWhatsOnData[1] = "";
 
function SiriusTuner() {
	log("--> SiriusTuner()");

	// user name and password for Sirius Radio Online
	this.user = preferences.user.value;
	this.password = preferences.password.value;

	// stores the cookies used for site authentication when retrieving data
	// from Sirius Radio online.
	this.cookies;
	this.cookieCount;
	this.token;
	this.captchaID;
	this.captchaImgID;
	this.captchaCode;
	this.canUseHighBitrate = null;

	// the windows media player COM object handle
	this.wmp = null;

	// config object, stores URLs for retrieving Sirius Radio data
	this.config;

	// array of available channels, stored as SiriusChannel objects
	this.channels;

	this.channelPlaying;

	// points to the first, current and last channels.  The SiriusChannel object
	// has nextChannel and previousChannel pointers, so together it makes
	// a linked list for easy navigation
	this.firstChannel;
	this.currentChannel;
	this.lastChannel;

	this.muteVolume = 0;
	this.pageName = "";
}

// singleton instance
SiriusTuner.instance = new SiriusTuner();

/**
 * Logs into Sirius Radio Online by posting the username and password to the
 * login servlet and storing the cookies received in the response.  Cookies
 * are returned even if the login fails, so this method counts on more than 1
 * cookie to be returned or it throws an error.
 */
SiriusTuner.prototype.login = function() {
	log("Logging in to sirius.com");
	
	//try logging in with old session cookies and token before wasting another daily login
	if (this.loginOldSession()) {
		log("Logged in using cached credentials.");
		return;
	}

	this.getToken();
	log("Token: " + this.token);
	log("CaptchaID: " + this.captchaID);
	log("CaptchaImgID: " + this.captchaImgID);

	if (this.token == null || this.captchaID == null || this.captchaImgID == null) {
		this.cookies = null;
		this.cookieCount = 0;
		log("An error occured retrieving token or captcha code or SIRIUS is currently unavailable. Please try again later.");
		throw new Error("An error occured retrieving token or captcha code or SIRIUS is currently unavailable. Check your firewall settings and try again.");
	}
	
	var codes = new Array("vRLCHr","Rk9f3b","tN2R1A","R3iwj5","jBjsvj","v3jvKg","IImNmx","cahMYf","Vw3rxG","R7KPgK","RUyTUS","cef11w","NAQbyX","q6EYAH","tReWYs","fimQlm","U6qsi6","m5Wkwh","FpVR2T","CuAF1k","sgnUw7","4N1RPP","ech2am","CtbsNQ","kXrPES","1AgXSR","5DHYSR","e3ru7T","c1yjHE","FR1ltI","Xtn36U","DHEWnx","8KePqv","1TKVVk","BIY138","RA6c83","SaluKT","T89gGV","gUPVqL","J4F3gi","BbQnjy","qLrRgi","c3eSfa","yAhdN5","3YW4WC","mPvBah","UZnHN4","x24GCx","GLdYdn","DsUIMk","7GCaEc","1WXPNr","SRpRsG","vSlae4","r95Vhm","1tGuk7","wnZyD4","c8lj6k","sdQ3X4","5FNMsi","Up7Rni","csjyJa","9Uq5rm","p9kbvj","Cy1iip","mc7y2c","SE3rqi","YmJ3Tv","Qr32YN","13rcdJ","xn33VA","tjxuf4","3hLBuU","3fntSq","rMYmpH","yvKfyR","bkxHDW","EtUSs3","3gA7wG","Yn3uUL","hcw9cg","aLI1R7","wmkRRP","Rm3C3i","CgS98N","xaF7cd","ATxch8","8I1rDk","C8896y","SiNusq","AQZ3kR","ARFUSP","hDgs72","Lxbg1X","4716A3","gCkAqa","wRDWeN","h64fGf","Cr2VPm","66SiiF");
	this.captchaCode = codes[this.captchaImgID-1];
	log("CaptchaCode: " + this.captchaCode);

	var loginURL = new URL();
	var postDataType = isEmailAddress(this.user) ? "guest" : "subscriber";
	var postDataPasswordName = isEmailAddress(this.user) ? "encryptedPassword" : "password";
	log("isEmailAddress(" + this.user +  "): " + isEmailAddress(this.user));

	//post data to be sent with login request... the encrypted password is
	//derived from the RSA Data Security, Inc. MD5 Message-Digest Algorithm

	loginURL.postData = "activity=login&type=" + postDataType + "&loginForm=" + postDataType + "&token=" + this.token + "&playerType=full&username=" + this.user + "&" + postDataPasswordName + "=" + hex_md5(this.password) + "&rememberMe=no&captchaID=" + this.captchaID + "&captcha_response=" + this.captchaCode;
	loginURL.setRequestHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0)");
	loginURL.setRequestHeader("Cookie", this.cookies);

	loginURL.fetch(servlet_url + "Login/" + postDataType);

	if (loginURL.response > 0 && loginURL.response < 400) {
		// got a valid response
		var cookies = loginURL.getResponseHeaders("Set-Cookie");
		if (cookies != null) {
			log("Received " + cookies.length + " cookies");

			if (cookies.length > 6) {
				this.setCookies(cookies);
				cookies = this.cookies.split(";");
				for (var i = 0; i < cookies.length; i++) {
					var evalStr = "preferences.cookie" + i + ".value = \"" + trim(cookies[i]) + "\";"
					log(evalStr);
					eval(evalStr);
				}
				preferences.token.value = this.token;
				savePreferences();

				if (this.loginOldSession()) {
					log("Logged in new credentials.");
					return;
				} else {
					savePreferences();
					var errorMsg = "Login failed: " + this.pageName;
					this.pageName = "";
					throw new Error(errorMsg);
				}
			} else {
				throw new Error("Login failed: Verify username/password and try again.");
			}
		} else {
			throw new Error("Login failed: no cookies returned");
		}
	} else {
		// got an error on the http level
		throw new Error("Login failed: " + loginURL.result);
	}
}

SiriusTuner.prototype.loginOldSession = function() {
	var cookies = new Array();
	for (var i = 0; i <= 10; i++)  {
		eval("cookies[" + i + "] = preferences.cookie" + i + ".value;");
	}
	if (isEmailAddress(this.user)) {
		cookies[10] = preferences.cookie10.value;
		cookies[11] = preferences.cookie11.value;
	}

	var token = preferences.token.value;

	for (var i = 0; i <= 10; i++) {	//cookies 10, 11 for guest only will be null for subscriber login.
		if (cookies[i] == null || cookies[i] == "" || cookies[i] == "null" || token == null || token == "" || token == "null")
			return false;
	}

	var playerURL = new URL();
	playerURL.location = servlet_url + "?stream=undefined";
	playerURL.setRequestHeader("Cookie", cookies.join(";"));
	playerURL.fetch();

	var resp = playerURL.responseData;

	var searchString = "SIRIUS Player - 100% Commercial-free Music";
	if (resp.indexOf(searchString) > -1) {
		this.canUseHighBitRate = HighBitrateEnabled(resp);
		log("High Bitrate Enabled: " + this.canUseHighBitRate);
		this.cookieCount = cookies.length;
		this.cookies = cookies.join("; ");
		this.token = token;
		return true;
	} else {
		log("Cached login failed:\n" + resp);
		this.cookies = null;
		this.cookieCount = 0;
		this.token = null;
		searchString = "var s_pageName = \"";
		if (resp.indexOf(searchString) > -1) {
			this.pageName = ExtractBetween(searchString, "\"", resp);
		}
		return false;
	}
}

SiriusTuner.prototype.getToken = function() {
	var url = new URL();
	url.location = servlet_url;
	url.setRequestHeader("User-Agent", "Mozilla/4.0 (compatible; MSIE 6.0)");
	url.fetch();

	var cookies = url.getResponseHeaders("Set-Cookie");
	this.setCookies(cookies);

	var resp = url.responseData;
	var searchString = "<input type=\"hidden\" name=\"token\" value=\"";
	var startIndex = resp.indexOf(searchString) + searchString.length;
	if (startIndex == searchString.length - 1) {
		this.token = null;
	} else {
		var length = resp.indexOf("\"", startIndex) - startIndex;
		this.token = resp.substring(startIndex, startIndex + length);
	}
	
	var searchString = "<input type=\"hidden\" name=\"captchaID\" value=\"";
	var startIndex = resp.indexOf(searchString) + searchString.length;
	if (startIndex == searchString.length - 1) {
		this.captchaID = null;
	} else {
		var length = resp.indexOf("\"", startIndex) - startIndex;
		this.captchaID = resp.substring(startIndex, startIndex + length);
	}
	
	var searchString = "captcha/image/img_";
	var startIndex = resp.indexOf(searchString) + searchString.length;
	if (startIndex == searchString.length - 1) {
		this.captchaImgID = null;
	} else {
		var length = resp.indexOf(".", startIndex) - startIndex;
		this.captchaImgID = parseInt(resp.substring(startIndex, startIndex + length), 10);
	}
}

SiriusTuner.prototype.GetLoginSuccess = function(responseData) {
	if (responseData.indexOf("MP: Launch Player") > -1) {
		log("GetLoginSuccess: True");
		return true;
	} else {
		log("GetLoginSuccess: False");
		var searchString = "var s_pageName = \"MP: ";
		log("indexOf " + searchString + ": " + responseData.indexOf(searchString));
		if (responseData.indexOf(searchString) > -1) {
			log(ExtractBetween(searchString, " -", responseData));
			this.pageName = ExtractBetween(searchString, " -", responseData);
		}
		return false;
	}	
}

SiriusTuner.prototype.isLoggedIn = function() {
	// we're logged in if cookies are set and there are more than six
	return (this.cookies != null && this.cookieCount > 6)
}

/**
 * Stores the cookies, converting the array of cookies received in the HTTP
 * response into a string ready for passing in HTTP requests.
 */
SiriusTuner.prototype.setCookies = function(cookies) {
	this.cookieCount = cookies.length;
	for (var i = 0; i < cookies.length; i++) {
		cookies[i] = cookies[i].substring(0, cookies[i].search(";"));
	}
	this.cookies = cookies.join("; ");
}

/**
 * Loads channel data by retrieving it from Sirius Radio Online and storing it
 * as SiriusChannel objects.
 */
SiriusTuner.prototype.loadChannelData = function() {
	var stations = this.getStations(this.getCategories());
	this.setChannelData(stations);
	Player.instance.updateNumberButtonTooltips();
}

SiriusTuner.prototype.setChannelData = function(channelData) {
	this.channels = new Array();
	if (channelData != null) {
		var num = 0;

		for (var i = 0; i < channelData.length; i++) {
			num = channelData[i].num - 0;
			this.channels[num] = channelData[i];
			if (i == 0) {
				this.firstChannel = this.channels[num];
				previousChannel = this.firstChannel;
			} else {
				this.channels[num].previousChannel = previousChannel;
				this.channels[num].previousChannel.nextChannel = this.channels[num];
				previousChannel = this.channels[num];
			}
		}

		this.lastChannel = this.channels[num];
		
		this.firstChannel.previousChannel = this.lastChannel;
		this.lastChannel.nextChannel = this.firstChannel;
	}
}

SiriusTuner.prototype.loadWhatsOn = function() {
	var whatsOnURL = new URL();
	var src = preferences.whatsOnDataURL.value;
	src = src + (getFilename(src).indexOf("?") > 0 ? (getFilename(src).indexOf("?") < getFilename(src).length-1 ? "&" + random() : random()) : "?" + random());
	//log("Retrieving whats on data from " + src);
	whatsOnURL.location = src
	whatsOnURL.fetchAsync(receiveWhatsOnData);
}

function receiveWhatsOnData(url) {
	log("Received What's On Data. Response: " + url.response);
	
	if (url.response > 0 && url.response < 400) {
		var result = url.result;

		if (previousWhatsOnData[1] == result) {
			log("Duplicate What's On results detected... checking count... " + previousWhatsOnData[0]);
			if (previousWhatsOnData[0] >= 4) {
				SiriusTuner.instance.advanceWhatsOnURL("Duplicate Results " + (previousWhatsOnData[0] + 1) + " times");
			} else {
				previousWhatsOnData[0]++;
			}
		} else {
			previousWhatsOnData[1] = result;
			previousWhatsOnData[0] = 0;
			var re = /^\d{1,2}\/\d{1,2}\/(\d{2}|\d{4}) \d{1,2}:\d{2}:\d{2} (AM|PM)$/;
			
			var MyStreams = null;
			if (result.indexOf("MyStreams") > -1) {	//handle what's on data in the form of JS.
				var setWhatsOnData = "MyStreams = " + ExtractBetween("MyStreams = ", ");", result) + ");";
				try {
					eval(setWhatsOnData);
				} catch (e) {
					SiriusTuner.instance.advanceWhatsOnURL(e);
				}
			} else if (re.test(result.split(String.fromCharCode(13))[0])) { //handle Stream_ON data format
				result = result.substr(result.indexOf(String.fromCharCode(13)) + 1);
				var crlf = String.fromCharCode(13) + String.fromCharCode(10);
				var channels = result.split(crlf + "---" + crlf);
				MyStreams = new Array();
				for (var i = 0; i < channels.length; i++) {
					var lines = channels[i].split(crlf);
					var data = new Array(parseInt(lines[0]), lines[2], lines[3], "");
					MyStreams[i] = data;
				}
			} else if (false) {
				
			}
			try {
				if (MyStreams != null && MyStreams.length > 0) {
					SiriusTuner.instance.setWhatsOn(MyStreams);
					Player.instance.updateScreen();
				} else {
					SiriusTuner.instance.advanceWhatsOnURL("Invalid Channel Data");
				}
			} catch(e) {
				SiriusTuner.instance.advanceWhatsOnURL(e);
			}
		}
	} else {
		SiriusTuner.instance.advanceWhatsOnURL("URL Response " + url.response);		
	}
}

SiriusTuner.prototype.getWhatsOnSources = function() {
	log("Retrieving What's On Sources...");
	var url = new URL();
	url.location = "http://www.neoscrib.com/neoscrib/datasources.php";
	url.fetchAsync(receiveDataSources);
}

function receiveDataSources(url) {
	if (url.response > 0 && url.response < 400) {
		var resp = url.responseData;
		try {
			var doc = XMLDOM.parse(resp);
			var e = doc.getElementsByName("url");	
			if (e.length > 0) {
				urls = new Array();
				for (var i = 0; i < e.length; i++) {
					urls[i] = e.item(i).firstChild.data;
				}
			}
		} catch (e) {
			//Player.instance.mainScreen.setStatusDisplay(e);
		}
	}
}

SiriusTuner.prototype.advanceWhatsOnURL = function(reason) {
	var advanced = false;
	log("Advancing What's On URL" + (reason != null ? " due to " + reason : ""));
	if(preferences.circleData.value == 1) {
		//log("Advancing What's On URL" + (reason != null ? " due to " + reason : ""));
		for (var i = 0; i < urls.length; i++) {
			if (preferences.whatsOnDataURL.value == urls[i]) {
				preferences.whatsOnDataURL.value = urls[(((i % urls.length) + 1) % urls.length)];
				advanced = true;
				savePreferences();
				break;
			}
		}
		if (!advanced && urls.length > 0)
			preferences.whatsOnDataURL.value = urls[0];
		log("What's On URL Changed to " + preferences.whatsOnDataURL.value);
	} else {
		log("Advancement of What's On URL disabled by user.");
	}
}

SiriusTuner.prototype.setWhatsOn = function(whatsOnData) {
	if (preferences.useSiriusBackstageData.value == 1 && this.channels != null) {
		for (var i = 0; i < whatsOnData.length; i++) {
			var num = whatsOnData[i][0];
			var author = whatsOnData[i][1];
			var title = whatsOnData[i][2];
			
			if (this.channels[num] != null) {
				var changed = false;
				if (this.channels[num].currentTitle == null || this.channels[num].currentTitle.indexOf(title) == -1) {
					this.channels[num].currentTitle = title;
					changed = true;
				}
				if (this.channels[num].currentArtist == null || this.channels[num].currentArtist.indexOf(author) == -1) {
					this.channels[num].currentArtist = author;
					changed = true;
				}

				if (changed) {
					this.checkAlerts(this.channels[num]);
					if (this.currentChannel != null && this.currentChannel.num == num)
						setMessengerStatus(this.currentChannel);
				}
			}
		}
	}
}

SiriusTuner.prototype.checkAlerts = function(channel) {
	if (preferences.alertEnable.value == 1) {
		if ((channel != null) && (this.currentChannel != channel) && (channel.currentArtist != null)) {
			var items = preferences.alertItem1.value + "::" + preferences.alertItem2.value + "::" + preferences.alertItem3.value + "::" + preferences.alertItem4.value + "::" + preferences.alertItem5.value;
			var alertItems = items.split("::");

			for (var i = 0; i < alertItems.length; i++) {
				alertItems[i] = trim(alertItems[i]);
				if (alertItems[i] != null && trim(alertItems[i].split("by:").join("").split("%").join("")).length >= 3) {
					var elem = alertItems[i].split("by:")
					elem[0] = trim(elem[0]);
					elem[elem.length-1] = trim(elem[elem.length-1]);
					if (like(elem[0], channel.currentTitle)) {
						if (elem.length > 1) {
							if (like(elem[1], channel.currentArtist))
								this.DoAlert(channel);
						} else {
							this.DoAlert(channel);
						}
					}
				}
			}
		}
	}
}

SiriusTuner.prototype.DoAlert = function(channel) {
	if (preferences.alertBeep.value == 1)
		beep()
	if (preferences.alertFocus.value == 1)
		focusWidget();
	if (preferences.alertAutoTune.value == 1) {
		Player.instance.playChannel(channel, false);
		Player.instance.showMainScreen();
	} else {
		Player.instance.displayArtistAlert(channel);
	}
}

SiriusTuner.prototype.initPlayer = function() {
	volume = preferences.volume.value;
	if (system.platform == "windows") {
		this.wmp = COM.createObject("WMPlayer.OCX");
		COM.connectObject(this.wmp, "wmp");
	}
	this.setVolume(volume);	
}

SiriusTuner.prototype.setCurrentChannel = function(channel) {
	// convert channel number to channel object
	log("setCurrentChannel Channel Type: " + typeof channel);
	if (typeof channel == "number" || typeof channel == "string") {
		var num = channel;
		channel = this.channels[num];
		if (channel == null) {
			throw new Error("Channel not found: " + num);
		}
	}
	
	this.currentChannel = channel;
	if (Player.instance.activeScreen instanceof ChannelGuideScreen) {
		Player.instance.activeScreen.setSelectedChannel(channel);
	}
	return channel;
}

SiriusTuner.prototype.play = function() {
	if (this.currentChannel.num != this.channelPlaying || (system.platform == "windows" && (this.wmp.playState == 10 || this.wmp.playState == 1))) {
		this.currentChannel.play(this.cookies);
		this.channelPlaying = this.currentChannel.num;
	}
}

SiriusTuner.prototype.stop = function() {
	this.channelPlaying = null;
	if (system.platform == "windows") {
		this.wmp.controls.stop();
	} else {
		log("Sending Stop Command: " + appleScript("tell application \"Windows Media Player\" to quit\n"));
	}
}

SiriusTuner.prototype.nextChannel = function () {
	if (this.currentChannel == null || this.currentChannel.nextChannel == null) {
		this.setCurrentChannel(this.firstChannel);
	} else {
		this.setCurrentChannel(this.currentChannel.nextChannel);
	}
}

SiriusTuner.prototype.previousChannel = function () {
	if (this.currentChannel == null || this.currentChannel.previousChannel == null) {
		this.setCurrentChannel(this.lastChannel);
	} else {
		this.setCurrentChannel(this.currentChannel.previousChannel);
	}
}

SiriusTuner.prototype.isStopped = function() {
	return (this.wmp.playState == 1);
}

SiriusTuner.prototype.getVolume = function() {
	if (system.platform == "windows") return this.wmp.settings.volume;
	return ((system.volume * 10) / 160) * 100
}

SiriusTuner.prototype.setVolume = function(volume) {
	if (volume < 0) volume = 0;
	if (volume > 100) volume = 100;
	if (system.platform == "windows") {
		this.wmp.settings.volume = volume;
	} else {
		system.volume = ((volume / 100) * 16);
	}
	preferences.volume.value = volume;
	savePreferences();
}

SiriusTuner.prototype.mute = function() {
	if (this.muteVolume == 0) {
		this.muteVolume = this.getVolume();
		this.setVolume(0);
		Player.instance.muted = true;
	} else {
		this.setVolume(this.muteVolume);
		this.muteVolume = 0;
		Player.instance.muted = false;
	}
}

SiriusTuner.prototype.getReceptionQuality = function () {
	if (system.wireless.powered == 1) {
		return system.wireless.signal;
	} else {
		return 100;
	}
}

function playChannel(url) {
	var pattern = /<PARAM NAME="FileName" VALUE="(.*?)">/;
	var matches = pattern.exec(url.result);
	try {
		var murl = matches[1];
	} catch (e) {
		Player.instance.stopButtonClicked();
		Player.instance.mainScreen.setStatusDisplay("Sirius is currently unavailable. Try again later.");
		return;
	}
	var urlArray = servlet_url.split("/");
	var domain = urlArray[0] + "//" + urlArray[2];

	if (murl.indexOf(domain) < 0) {
		murl = domain + murl;
	}
	
	if (matches != null) {
		log("Loading Stream: " + murl);
		SiriusTuner.instance.currentChannel.url = murl;
		if (system.platform == "windows" && SiriusTuner.instance.wmp.URL != murl) {
			SiriusTuner.instance.wmp.URL = murl;
			SiriusTuner.instance.wmp.controls.play();
		} else if (system.platform != "windows") {
			var script = 'tell application "Windows Media Player" to open location "' + murl + '"\n';
			var result = appleScript("tell application \"Windows Media Player\" to open location \"" + murl + "\"\ntell app \"System Events\"\nset visible of process \"Windows Media Player\" to false\nend tell", 20);
			if (result.indexOf("AppleScript Error") >= 0 && preferences.macEnableWMPDownloadAlert.value == 1)
			{
				result = alert("Windows Media Player is required to use this Widget.\nDo you want to open a browser for downloading?", "Yes", "No", "No, don't ask me again");
				if (result == 1) openURL("http://www.microsoft.com/windows/windowsmedia/player/mac/mp9/default.aspx");
				if (result == 3) preferences.macEnableWMPDownloadAlert.value = 0;
			}
		}
	} else {
		log("No stream Matches! Checking pageName...");
		pattern = /var s_pageName = "(.*?)"/;
		matches = pattern.exec(url.result);
		if (matches != null) {
			log("pageName: " + matches[1]);
			Player.instance.mainScreen.setStatusDisplay(matches[1] + ".");
		} else {
			log("No pageName! Reload Widget.");
			Player.instance.mainScreen.setStatusDisplay("An unknown error occured. Try reloading this Widget.");
		}
	}
}